var POSTAL_C_ID = "#postalCode";
var AMOUNT_ID = "#payAmount";
var ADDRESS_ID = "#address";

var serviceUrlTest = "http://localhost:8080/Services/stores";
var serviceUrl = "https://api.openpay.mx/stores";
var markers = [];
var geocoder;
var map;
var originMarker;
var destiny;

var markerArray = [];
var directionsService;
var directionsDisplay;
var stepDisplay;

var oms;

$( window ).on( "load", initScreen);
$( "#locationButton" ).on("click", findInCurrentLocation);
$( "#geocodeButton" ).on("click", function() {
      geocodeAddress(geocoder);
});

function initMap() {
  geocoder = new google.maps.Geocoder();
  map = new google.maps.Map(document.getElementById('map'), {
    zoom: 16,
    center: {lat: 19.432603, lng: -99.133276},
    streetViewControl: false
  });
  google.maps.event.addListener(map, "dblclick", function(event){
    placeMarker(event.latLng);
  });
  oms = new OverlappingMarkerSpiderfier(map, { 
    markersWontMove: true, 
    markersWontHide: true,
    basicFormatEvents: true
  });
  
 directionsService = new google.maps.DirectionsService;
 directionsDisplay = new google.maps.DirectionsRenderer({map: map});
 stepDisplay = new google.maps.InfoWindow;
}

function showLoad(){
  $("#loading-panel").fadeIn();
}

function hideLoad(){
  $("#loading-panel").fadeOut();
}

function initScreen() {
  setAmountParam();
  setAddressParams();
  setLocationParam();
}

function setLocationParam(){
  var isLocationNotAllowed = $.url().param('locationNotAllowed');
  if (!isLocationNotAllowed) {
    findInCurrentLocation();
  }
}

function setAddressParams(){
  var address = $.url().param('address');
  if (!isEmpty(address)) {
    $(ADDRESS_ID).val(address);
  }
  var cp = $.url().param('postalCode');
  if (!isEmpty(cp)) {
    try{
      cp= parseInt(cp)
    }catch(e){
      cp='';
    }
    $(POSTAL_C_ID).val(cp);
  }
}

function setAmountParam(){
  var amount = $.url().param('amount');
  try{
    if(isEmpty(amount)){
       throw 'monto vacio';
    }else{
      amount = parseInt(amount);
    }
  }catch(e){
    amount = 1;
  }
  $(AMOUNT_ID).val(amount);
}

function findInCurrentLocation() {
  showLoad();
  if(navigator.geolocation) {
    navigator.geolocation.getCurrentPosition(showPosition,handleError);
  }
}

function handleError(error)
{
	switch (error.code)
	{
		case error.PERMISSION_DENIED:
			console.log('Ubicación no permitida');
      $("#locationButton").fadeOut();
      $( "#floating-panel" ).animate({
        "max-width": "610px",
      }, 500);
			break;
		case error.POSITION_UNAVAILABLE:
			console.log('Ubicación no disponible.');
			break;
		case error.PERMISSION_DENIED_TIMEOUT:
			console.log('Usurario no permitio o denego su ubicación');
      $("#locationButton").fadeOut();
      $( "#floating-panel" ).animate({
        "max-width": "610px",
      }, 500);
			break;
		case error.UNKNOWN_ERROR:
			console.log('Un error ocurrio durante la optencion de la ubicación')
			break;
	}
    hideLoad();
}

function findInCurrentProyection() {
  clearMarkers();
  setTimeout(findStores,1000);
}

function findStores() {
  var center = map.getBounds().getCenter();
  var amount = $(AMOUNT_ID).val();
  try{
    if(isEmpty(amount)){
      throw 'monto vacio';
    }
    amount = parseInt(amount);
    showStores(getRadius(), amount, center.lat(), center.lng());  
  }catch(e){
    alert("Ingresa una cantidad numerica en el campo monto");
  }
}

function showStores(radius, amount, lat, lng){
  var filter = {
    kilometers: radius,
    amount: amount,
    latitud: lat,
    longitud: lng,
    limit: 500
  };
  var urlService = serviceUrlTest;
  var isLocalTest = $.url().param('isLocalTest');
  if (!isLocalTest) {
    urlService = serviceUrl;
  }
  $.get(urlService, filter, function(data, status){
    if(data===null||data.length===0){
      alert("No se encontraron tiendas cercanas.\nAmplia el rango del mapa y tendras mas suerte!.");
      return;
    }
    for (var i = 0; i < data.length; i++) {
      addMarker(data[i], i * 100,i);
    }
  }).fail(function() {
    alert( "Error en el servidor" );
  });
}

function geocodeAddress(geocoder) {
  var address = $(ADDRESS_ID).val();
  var postalCode = $(POSTAL_C_ID).val();
  var restrictions = null;
  var filter = null;
  if(postalCode === '' && address === ''){
    alert("Ingresa tu direccion o tu código postal");
    return;
  }
  restrictions = {
    'country': 'MX',
  };
  if(postalCode !== ''){
     address = address +", "+postalCode;
  }
  if(address === ''){
    filter = {
      'componentRestrictions': restrictions
    };
  }else{
    filter = {
      'address': address,
      'componentRestrictions': restrictions
    };
  }
  findPlace(filter, geocoder);
}

function findPlace(filter, geocoder){
  geocoder.geocode(filter, function(results, status) {
    var isOk = false;
    if (status === google.maps.GeocoderStatus.OK) {
      map.setCenter(results[0].geometry.location);
      var marker = getMyMarker(results[0].geometry.location);
      clearAllMap();
      setTimeout(findStores,1000);
      markers.push(marker);
    } else {
      alert('El lugar no fue encontrado');
    }
  });
}

function placeMarker(location) {
  clearAllMap();
  originMarker = getMyMarker(location);
  map.panTo(location);
  setTimeout(findStores,1000);
}

function getMyMarker(location) {
  var marker = new google.maps.Marker({
      position: location, 
      icon: "https://s3.amazonaws.com/public.openpay.mx/mapa-tiendas/img/person.png",
      draggable:true,
      map: map
  });
  google.maps.event.addListener(marker, 'dragend', function(event){
        placeMarker(event.latLng);
  });
  return marker;
}

function showPosition(position) {
  clearAllMap();
  var location = new google.maps.LatLng(
    position.coords.latitude, 
    position.coords.longitude);
  map.setCenter(location);
  var marker = getMyMarker(location);
  originMarker = marker;
  setTimeout(findStores,1000);
  originMarker = marker;
  hideLoad();
}


function getRadius(){
  var bounds = map.getBounds();
  var northEast = bounds.getNorthEast();
  var center = bounds.getCenter();
  var point1 = new google.maps.LatLng(center.lat(), northEast.lng());
  var point2 = new google.maps.LatLng(northEast.lat(), center.lng());
  var d1 = getDistance(point1, center);
  var d2 = getDistance(point2, center);
  if(d1<d2){
    return d1;
  }else{
    return d2;
  }
}

function getDistance(p1, p2){
  return (google.maps.geometry.spherical.computeDistanceBetween(p1, p2) / 1000).toFixed(2);
}

function addMarker(store, timeout,id) {
  var position = new google.maps.LatLng(store.geolocation.lat, store.geolocation.lng);
  /*var image = null;
  if (!isEmpty(store.paynet_chain.thumb)){
    image = store.paynet_chain.thumb;
  }*/
  window.setTimeout(function() {
    var marker = new google.maps.Marker({
      position: position,
   //   icon: image,
      animation: google.maps.Animation.DROP
    });
    attachContent(marker, getMarkerContent(store, marker, id));
    markers.push(marker);
  }, timeout);
}

function getMarkerContent(store, marker, id){
  var direccion = "";
  if (store.address.line1!==null) {
    direccion += store.address.line1;
  }
  if (store.address.line2!==null) {
    direccion += ", " + store.address.line2 ;
  }
  if (store.address.line3!==null) {
    direccion += ", " + store.address.line3;
  }
  var city= "";
  if (store.address.city!==null) {
    city = store.address.city;
  }
  var state= "";
  if (store.address.state!==null) {
    state = store.address.state;
  }
  var postal_code = "";
  if (store.address.postal_code!==null) {
    postal_code = store.address.postal_code;
  }
  var divLogo = "   <div class='divLogo'/></div>";
  if (store.paynet_subchain != undefined && store.paynet_subchain != null && !isEmpty(store.paynet_subchain.url_logo)){
    divLogo = "   <div class='divLogo' style=\"background-image: url('"+store.paynet_subchain.url_logo+"');\"/></div>";
  } else if (!isEmpty(store.paynet_chain.logo)){
    divLogo = "   <div class='divLogo' style=\"background-image: url('"+store.paynet_chain.logo+"');\"/></div>";
  }
  var name = store.paynet_chain.name;
  if (store.paynet_subchain != undefined && store.paynet_subchain != null && !isEmpty(store.paynet_subchain.name)) {
    name = store.paynet_subchain.name;
  }

  return "<div class='infoWindow'>"+
  "   <div class='infoElement'>"+
  "     <strong style='font-size: 16px; line-height: 140%;'>" + store.paynet_chain.name + "</strong></br>" +
  "     <strong>Dirección:</strong> " + direccion + "</br>" +
  "     <strong>Ciudad:</strong> " + city + "</br>" +
  "     <strong>Estado:</strong> " + state + "</br>" +
  "     <strong>Código postal:</strong> " + postal_code +"</br>"+
  "   </div>"+
  divLogo +
  "   <div class='autoRuta' onclick='getRoute("+id+",1)' title='Llegar en auto'></div>"+
  "   <div class='pieRuta' onclick='getRoute("+id+",2)' title='Llegar a pie'></div>"+
  "</div>";
}

function attachContent(marker, html) {
  buildInfoWindow(marker, html);
}

function buildInfoWindow(marker,html) {
  var content = document.createElement("DIV");
  content.className ='infoContent';
  var title = document.createElement("DIV");
  var streetview = document.createElement("DIV");
  streetview.className ='panorama';
  title.innerHTML = html;
  title.style["margin-left"] = "4px";
  content.appendChild(title);
  content.appendChild(streetview);
  
  var infowindow = new google.maps.InfoWindow({
    content: content
  });

  google.maps.event.addListener(marker, 'spider_click', function(e) {  // 'spider_click', not plain 'click'
    infowindow.open(map, marker);
  });
  oms.addMarker(marker);
  
  /*google.maps.event.addListener(marker, "click", function() {
    infowindow.open(map, marker);
  });*/
  
  google.maps.event.addListenerOnce(infowindow, "domready", function() {
    var panorama = new google.maps.StreetViewPanorama(streetview, {
        navigationControl: false,
        enableCloseButton: false,
        addressControl: false,
        linksControl: false,
        clickToGo: true,
        visible: true,
        position: marker.getPosition()
    });
  });
}

function isEmpty(val) {
  return val===null||val===undefined||val==='';
}

function getRoute(id, mode){
  switch (mode) {
    case 1:
      mode = google.maps.DirectionsTravelMode.DRIVING;
      break;
    case 2:
      mode = google.maps.DirectionsTravelMode.WALKING;
      break;
  }
  var request = {
    origin: originMarker.getPosition(),
    destination: markers[id].getPosition(),
    travelMode: mode
  };
  clearRoute();
  directionsDisplay.setMap(map);
  directionsService.route(request, function(response, status) {
   /* if (status === google.maps.DirectionsStatus.OK) {
      document.getElementById('warnings-panel').innerHTML =
          '<b>' + response.routes[0].warnings + '</b>';
      directionsDisplay.setDirections(response);
      directionsDisplay.setOptions( { suppressMarkers: true } );
      showSteps(response, markerArray, stepDisplay, map);
    } else {
      window.alert('Directions request failed due to ' + status);
    }*/
   if (status == google.maps.DirectionsStatus.OK) {
      directionsDisplay.setDirections(response);
      directionsDisplay.setOptions( { suppressMarkers: true } );
    } else {
      window.alert('Ruta no encontrada');
    }
  });
}

function showSteps(directionResult, markerArray, stepDisplay, map) {
  var myRoute = directionResult.routes[0].legs[0];
  for (var i = 0; i < myRoute.steps.length; i++) {
    var marker = markerArray[i] = markerArray[i] || new google.maps.Marker;
    marker.setMap(map);
    marker.setPosition(myRoute.steps[i].start_location);
    marker.setIcon("https://s3.amazonaws.com/public.openpay.mx/mapa-tiendas/img/auto-marker.png");
    attachInstructionText(
        stepDisplay, marker, myRoute.steps[i].instructions, map);
  }
}

function attachInstructionText(stepDisplay, marker, text, map) {
  google.maps.event.addListener(marker, 'click', function() {
    stepDisplay.setContent(text);
    stepDisplay.open(map, marker);
  });
}

function clearAllMap(){
  clearRoute();
  clearMarkers();
  if (originMarker!=null) {
   originMarker.setMap(null);
  }
}

function clearRoute() {
  for (var i = 0; i < markerArray.length; i++) {
    markerArray[i].setMap(null);
  }
  directionsDisplay.setMap(null);
}

function clearMarkers() {
  for (var i = 0; i < markers.length; i++) {
    markers[i].setMap(null);
  }
  oms.removeAllMarkers();
  markers = [];
}
